@@ -1,16 +1,27 @@
 .onAttach <- function(libname, pkgname) {
   packageStartupMessage(
     "Welcome to echarts4r\n\n",
-    "Documentation: echarts4r.john-coene.com"
+    "Docs: echarts4r.john-coene.com"
   )
   
   shiny::registerInputHandler("echarts4rParse", function(data, ...) {
     jsonlite::fromJSON(jsonlite::toJSON(data, auto_unbox = TRUE))
   }, force = TRUE)
+  
+  options(
+    "ECHARTS4R_THEME" = NULL,
+    "ECHARTS4R_FONT_FAMILY" = NULL
+  )
+  
 }
 
 .onLoad <- function(libname, pkgname) {
   shiny::registerInputHandler("echarts4rParse", function(data, ...) {
     jsonlite::fromJSON(jsonlite::toJSON(data, auto_unbox = TRUE))
   }, force = TRUE)
+  
+  options(
+    "ECHARTS4R_THEME" = NULL,
+    "ECHARTS4R_FONT_FAMILY" = NULL
+  )
 }
\ No newline at end of file
